use xcb;
use ui::font;

use std::rc::Rc;
use std::sync::Arc;

pub trait Widget {
    fn init(&mut self) {}
    fn render(&mut self, x: u16);
    fn width(&mut self) -> u16;
    fn handle_event(&mut self, _event: &xcb::GenericEvent, _is_finishing: bool) -> bool {
        false
    }
    fn finish(&mut self) {}
}

pub struct DrawContext {
    conn: Arc<xcb::Connection>,
    picture: xcb::render::Picture,
    pen: xcb::render::Picture,
    fonts: Rc<font::FontLoader>,
    bg_color: xcb::render::Color
}

impl DrawContext {
    pub fn new(conn: Arc<xcb::Connection>, picture: xcb::render::Picture, fonts: Rc<font::FontLoader>) -> DrawContext {
        let pen = conn.generate_id();
        let color = xcb::render::Color::new(0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF);
        xcb::render::create_solid_fill(
            &conn,
            pen,
            color
        );

        DrawContext {
            conn: conn,
            picture: picture,
            pen: pen,
            fonts: fonts,
            bg_color: xcb::render::Color::new(0, 0, 0, 0xFFFF)
        }
    }

    pub fn draw_bg(&self, x: u16, width: u16) {
        let color = xcb::render::Color::new(
            self.bg_color.red(),
            self.bg_color.green(),
            self.bg_color.blue(),
            self.bg_color.alpha()
        );
        xcb::render::fill_rectangles(
            &self.conn,
            xcb::render::PICT_OP_SRC as u8,
            self.picture,
            color,
            &[xcb::Rectangle::new(x as i16, 0, width, 20)]
        );
    }

    pub fn measure_text(&self, name: &str) -> u16 {
        let text = self.fonts.create_renderable_text(name);
        text.width
    }

    pub fn draw_text(&self, name: &str, x: u16) {
        let text = self.fonts.create_renderable_text(name);
        let baseline = 20 - self.fonts.default_offset(20);
        text.render(&self.conn, self.pen, self.picture, x, baseline as u16);
    }
}
